@@ -12,13 +12,16 @@
 
 import os
 import sys
+import time
+import signal
 import shutil
 import logging
 import optparse
 import tempfile
+import subprocess
+import warnings
 
 from salttesting import TestLoader, TextTestRunner
-from salttesting.ext.HTMLTestRunner import HTMLTestRunner
 try:
     from salttesting.ext import console
     width, height = console.getTerminalSize()
@@ -56,9 +59,21 @@ def print_header(header, sep='~', top=True, bottom=True, inline=False,
 
 
 class SaltTestingParser(optparse.OptionParser):
+    support_docker_execution = False
     support_destructive_tests_selection = False
+    source_code_basedir = None
 
     def __init__(self, testsuite_directory, *args, **kwargs):
+        if kwargs.pop('html_output_from_env', None) is not None or \
+                kwargs.pop('html_output_dir', None) is not None:
+            warnings.warn(
+                'The unit tests HTML support was removed from {0}. Please '
+                'stop passing \'html_output_dir\' or \'html_output_from_env\' '
+                'as arguments to {0}'.format(self.__class__.__name__),
+                category=DeprecationWarning,
+                stacklevel=2
+            )
+
         # Get XML output settings
         xml_output_dir_env_var = kwargs.pop(
             'xml_output_from_env',
@@ -72,19 +87,6 @@ class SaltTestingParser(optparse.OptionParser):
             )
         )
 
-        # Get HTML output settings
-        html_output_dir_env_var = kwargs.pop(
-            'html_output_from_env',
-            'HTML_TESTS_OUTPUT_DIR'
-        )
-        html_output_dir = kwargs.pop('html_output_dir', None)
-        self.html_output_dir = os.environ.get(
-            html_output_dir_env_var,
-            html_output_dir or os.path.join(
-                tempfile.gettempdir(), 'html-tests-output'
-            )
-        )
-
         # Get the desired logfile to use while running tests
         self.tests_logfile = kwargs.pop('tests_logfile', None)
 
@@ -106,12 +108,20 @@ class SaltTestingParser(optparse.OptionParser):
                       'or removing users from your system for example. '
                       'Default: %default')
             )
+
+        if self.support_docker_execution is True:
+            self.test_selection_group.add_option(
+                '--docked',
+                default=None,
+                help='Run the tests suite in the chosen Docker container'
+            )
+
         self.test_selection_group.add_option(
             '-n',
             '--name',
             dest='name',
             action='append',
-            default=[],
+            default=None,
             help=('Specific test name to run. A named test is the module path '
                   'relative to the tests directory')
         )
@@ -139,15 +149,6 @@ class SaltTestingParser(optparse.OptionParser):
                     self.xml_output_dir
                 )
             )
-        if self.html_output_dir:
-            self.output_options_group.add_option(
-                '--html-out',
-                default=False,
-                action='store_true',
-                help='HTML test runner output(Output directory: {0})'.format(
-                    self.html_output_dir
-                )
-            )
         self.output_options_group.add_option(
             '--no-report',
             default=False,
@@ -183,6 +184,23 @@ class SaltTestingParser(optparse.OptionParser):
         )
         self.pre_execution_cleanup()
         self._validate_options()
+
+        if self.support_docker_execution and self.options.docked is not None:
+            if self.source_code_basedir is None:
+                raise RuntimeError(
+                    'You need to define the \'source_code_basedir\' attribute '
+                    'in {0!r}.'.format(self.__class__.__name__)
+                )
+            # No more processing should be done. We'll exit with the return
+            # code we get from the docker container execution
+            self.exit(self.run_suite_in_docker())
+
+        print('Current Directory: {0}'.format(os.getcwd()))
+        print_header(
+            'Test suite is running under PID {0}'.format(os.getpid()),
+            bottom=False
+        )
+
         self._setup_logging()
         return (self.options, self.args)
 
@@ -209,14 +227,6 @@ class SaltTestingParser(optparse.OptionParser):
                 'at {0!r}'.format(self.xml_output_dir)
             )
 
-        if self.html_output_dir is not None and self.options.html_out:
-            if not os.path.isdir(self.html_output_dir):
-                os.makedirs(self.html_output_dir)
-            print(
-                'Generated unit test HTML reports will be stored '
-                'at {0!r}'.format(self.html_output_dir)
-            )
-
         self.validate_options()
 
         if self.support_destructive_tests_selection:
@@ -224,12 +234,6 @@ class SaltTestingParser(optparse.OptionParser):
             # destructive tests should be executed or not.
             os.environ['DESTRUCTIVE_TESTS'] = str(self.options.run_destructive)
 
-        print('Current Directory: {0}'.format(os.getcwd()))
-        print_header(
-            'Test suite is running under PID {0}'.format(os.getpid()),
-            bottom=False
-        )
-
     def validate_options(self):
         '''
         Validate the provided options. Override this method to run your own
@@ -293,7 +297,7 @@ class SaltTestingParser(optparse.OptionParser):
         method.
         '''
         if self.options.clean is True:
-            for path in (self.xml_output_dir, self.html_output_dir):
+            for path in (self.xml_output_dir,):
                 if path is None:
                     continue
                 if os.path.isdir(path):
@@ -320,20 +324,6 @@ class SaltTestingParser(optparse.OptionParser):
                 verbosity=self.options.verbosity
             ).run(tests)
             self.testsuite_results.append((header, runner))
-        elif self.options.html_out:
-            runner = HTMLTestRunner(
-                stream=open(
-                    os.path.join(
-                        self.html_output_dir, 'report_{0}.html'.format(
-                            header.replace(' ', '_')
-                        )
-                    ),
-                    'w'
-                ),
-                verbosity=self.options.verbosity,
-                title=header,
-            ).run(tests)
-            self.testsuite_results.append((header, runner))
         else:
             runner = TextTestRunner(
                 stream=sys.stdout,
@@ -449,6 +439,107 @@ class SaltTestingParser(optparse.OptionParser):
         )
         self.exit(exit_code)
 
+    def run_suite_in_docker(self):
+        '''
+        Run the tests suite in a Docker container
+        '''
+        # Let's start the Docker container and run the tests suite there
+        if '/' not in self.options.docked:
+            container = 'salttest/{0}'.format(self.options.docked)
+        else:
+            container = self.options.docked
+
+        calling_args = ['/salt-source/tests/runtests.py']
+        for option in self._get_all_options():
+            if option.dest is None:
+                # For example --version
+                continue
+
+            if option.dest in ('docked', 'verbosity'):
+                # We don't need to pass the --docked argument inside the docker
+                # container, and verbose will be handled bellow
+                continue
+
+            default = self.defaults.get(option.dest)
+            value = getattr(self.options, option.dest, default)
+
+            if default == value:
+                # This is the default value, no need to pass the option to the
+                # parser
+                continue
+
+            if option.action.startswith('store_'):
+                calling_args.append(option.get_opt_string())
+
+            elif option.action == 'append':
+                for val in (value is not None and value or default):
+                    calling_args.extend(
+                       [option.get_opt_string(), str(val)]
+                )
+            elif option.action == 'count':
+                calling_args.extend(
+                    [option.get_opt_string()] * value
+                )
+            else:
+                calling_args.extend(
+                    [option.get_opt_string(),
+                    str(value is not None and value or default)]
+                )
+
+        if not self.options.run_destructive:
+            calling_args.append('--run-destructive')
+
+        if self.options.verbosity > 1:
+            calling_args.append(
+                '-{0}'.format('v' * (self.options.verbosity - 1))
+            )
+
+        print_header(
+            'Running the tests suite under the {0!r} docker container'.format(
+                container
+            )
+        )
+        call = subprocess.Popen(
+            ['docker',
+             'run',
+             '-v',
+             '{0}:/salt-source'.format(self.source_code_basedir),
+             '-w',
+             '/salt-source',
+             container,
+             ] + calling_args,
+            env=os.environ.copy(),
+            close_fds=True,
+        )
+
+        signalled = terminating = exiting = False
+
+        while True:
+            try:
+                time.sleep(0.15)
+                if exiting:
+                    break
+                elif terminating and not exiting:
+                    exiting = True
+                    call.kill()
+                    break
+                elif signalled and not terminating:
+                    terminating = True
+                    call.terminate()
+                else:
+                    call.poll()
+                    if call.returncode is not None:
+                        # Finshed
+                        break
+            except KeyboardInterrupt:
+                print('Caught CTRL-C, exiting...')
+                signalled = True
+                call.send_signal(signal.SIGINT)
+
+        call.wait()
+
+        self.exit(call.returncode)
+
 
 class SaltTestcaseParser(SaltTestingParser):
     '''
@@ -461,8 +552,6 @@ class SaltTestcaseParser(SaltTestingParser):
         self.option_groups.remove(self.test_selection_group)
         if self.has_option('--xml-out'):
             self.remove_option('--xml-out')
-        if self.has_option('--html-out'):
-            self.remove_option('--html-out')
 
     def get_prog_name(self):
         return '{0} {1}'.format(sys.executable.split(os.sep)[-1], sys.argv[0])
